package it.mulders.mcs.search.vulnerability;

import it.mulders.mcs.common.Result;
import it.mulders.mcs.search.vulnerability.ComponentReportResponse.ComponentReport.ComponentReportVulnerability;
import org.assertj.core.api.WithAssertions;
import org.junit.jupiter.api.DisplayNameGeneration;
import org.junit.jupiter.api.DisplayNameGenerator;
import org.junit.jupiter.api.Test;

@DisplayNameGeneration(DisplayNameGenerator.ReplaceUnderscores.class)
class ComponentReportResponseBodyHandlerTest implements WithAssertions {
    @Test
    void parse_component_report_with_vulnerabilities() {
        // Arrange
        var input = getClass().getResourceAsStream("/vulnerabilities-component-report-response.json");

        // Act
        var result = ComponentReportResponseBodyHandler.toComponentReportResponse(input);

        // Assert
        assertThat(result).isInstanceOf(Result.Success.class);
        var response = result.value();
        assertThat(response.componentReports()).hasSize(1);
        assertThat(response.componentReports()[0].coordinates())
                .isEqualTo("pkg:maven/org.apache.shiro/shiro-web@1.9.0");
        assertThat(response.componentReports()[0].reference())
                .isEqualTo("https://ossindex.sonatype.org/component/pkg:maven/org.apache.shiro/shiro-web@1.9.0");

        assertThat(response.componentReports()[0].vulnerabilities()).hasSize(2);
        assertThat(response.componentReports()[0].vulnerabilities())
                .contains(
                        new ComponentReportVulnerability(
                                "CVE-2022-40664",
                                "[CVE-2022-40664] CWE-287: Improper Authentication",
                                9.8,
                                "https://ossindex.sonatype.org/vulnerability/CVE-2022-40664?component-type=maven&component-name=org.apache.shiro%2Fshiro-web"));
        assertThat(response.componentReports()[0].vulnerabilities())
                .contains(
                        new ComponentReportVulnerability(
                                "CVE-2023-34478",
                                "[CVE-2023-34478] CWE-22: Improper Limitation of a Pathname to a Restricted Directory ('Path Traversal')",
                                9.8,
                                "https://ossindex.sonatype.org/vulnerability/CVE-2023-34478?component-type=maven&component-name=org.apache.shiro%2Fshiro-web"));
    }

    @Test
    void parse_component_report_with_no_vulnerabilities() {
        // Arrange
        var input = getClass().getResourceAsStream("/no-vulnerabilities-component-report-response.json");

        // Act
        var result = ComponentReportResponseBodyHandler.toComponentReportResponse(input);

        // Assert
        assertThat(result).isInstanceOf(Result.Success.class);
        var response = result.value();
        assertThat(response.componentReports()).hasSize(1);
        assertThat(response.componentReports()[0].coordinates())
                .isEqualTo("pkg:maven/org.codehaus.plexus/plexus-utils@3.4.1");
        assertThat(response.componentReports()[0].reference())
                .isEqualTo("https://ossindex.sonatype.org/component/pkg:maven/org.codehaus.plexus/plexus-utils@3.4.1");
        assertThat(response.componentReports()[0].vulnerabilities()).isEmpty();
    }
}
